"use strict";
/**
 * @athenna/core
 *
 * (c) João Lenon <lenon@athenna.io>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Ignite = void 0;
/**
 * Importing global files to be added to global module of Node.js
 */
require("@athenna/ioc");
require("@athenna/config/src/Utils/global");
const Application_1 = require("./Application");
const config_1 = require("@athenna/config");
const path_1 = require("path");
const utils_1 = require("@secjs/utils");
const DuplicatedApplicationException_1 = require("./Exceptions/DuplicatedApplicationException");
class Ignite {
    constructor(fileName) {
        /**
         * Change all process.cwd commands to return the
         * root path where the application root is stored.
         *
         * It keeps like that:
         * go out from athenna core src folder ->
         * go out from athenna core root folder ->
         * go out from athenna packages folder ->
         * go out from node_modules folder.
         *
         * Now process.chdir is in the application root.
         */
        if (!process.env.CORE_TESTING) {
            process.chdir(path_1.resolve(__dirname, '..', '..', '..', '..'));
        }
        this.resolveNodeTs(fileName);
        this.resolveNodeEnv();
    }
    async fire() {
        /**
         * Load all config files of config folder
         */
        await Config.load(utils_1.Path.config());
        this.clearConsole();
        /**
         * Using import because logger needs to be set after
         * resolveNodeEnv method has been called.
         */
        this.logger = utils_1.resolveModule(await Promise.resolve().then(() => __importStar(require('./Utils/Logger'))));
        const providers = await this.getProviders();
        await this.registerProviders(providers);
        await this.bootProviders(providers);
        await this.preloadFiles();
        return this.createApplication();
    }
    /**
     * Get the instance of the application inside of Ignite class.
     *
     * @return Application
     */
    getApplication() {
        if (!this.application) {
            throw new DuplicatedApplicationException_1.DuplicatedApplicationException();
        }
        return this.application;
    }
    /**
     * Create a new instance of application inside
     * Ignite class.
     *
     * @return Application
     */
    createApplication() {
        if (this.application) {
            throw new DuplicatedApplicationException_1.DuplicatedApplicationException();
        }
        this.application = new Application_1.Application(this.extension);
        return this.application;
    }
    /**
     * Resolve the NODE_TS Env variable verifying if the extension
     * of the file that is running Ignite class is .ts or other.
     *
     * @param fileName
     * @private
     */
    resolveNodeTs(fileName) {
        const { ext } = path_1.parse(fileName);
        if (ext === '.ts') {
            process.env.NODE_TS = 'true';
            this.extension = '.ts';
            return;
        }
        process.env.NODE_TS = 'false';
        this.extension = (ext || '');
    }
    /**
     * Resolve the NODE_ENV Env variable verifying if it's already
     * set. If not, get the content of config/app file and take the
     * environment key value to set as NODE_ENV. Then, resolve the
     * .env.${NODE_ENV} file in Node.js process.
     *
     * @private
     */
    resolveNodeEnv() {
        if (!process.env.NODE_ENV) {
            let env = new utils_1.File(utils_1.Path.config(`app${this.extension}`))
                .loadSync()
                .getContentSync()
                .toString()
                .split('environment:')[1]
                .split(',')[0]
                .trim();
            if (env.includes('process.env.NODE_ENV')) {
                env = env
                    .split('process.env.NODE_ENV')[1]
                    .replace(/'|"/g, '')
                    .replace(/\|/g, '')
                    .trim();
            }
            process.env.NODE_ENV = env;
        }
        config_1.resolveEnvFile();
    }
    /**
     * Clear the console if isn't in debug mode and NODE_ENV is
     * not set to testing environment.
     *
     * @private
     */
    clearConsole() {
        const isNotDebugModeOrTesting = !Env('APP_DEBUG') &&
            (Env('NODE_ENV') === 'test' ||
                Env('NODE_ENV') === 'testing' ||
                Env('BOOT_LOGS') === 'false');
        if (isNotDebugModeOrTesting) {
            return;
        }
        console.clear();
    }
    /**
     * Get all the providers from config/app file with export normalized.
     *
     * export default, export, module.exports, etc.
     *
     * @private
     */
    async getProviders() {
        const providers = await Promise.all(Config.get('app.providers'));
        const providersNormalized = [];
        providers.forEach(Provider => {
            providersNormalized.push(utils_1.resolveModule(Provider));
        });
        providersNormalized.forEach(p => this.logger.success(`Registering ${p.name}`));
        return providersNormalized;
    }
    /**
     * Boot all the providers calling the boot method
     * and reading the register attributes inside providers.
     *
     * @private
     */
    async bootProviders(providers) {
        const promises = providers.map(Provider => {
            const provider = new Provider();
            provider.registerAttributes();
            if (provider.boot)
                return provider.boot();
            return null;
        });
        await Promise.all(promises);
    }
    /**
     * Register all the providers calling the register method
     * and reading the register attributes inside providers.
     *
     * @private
     */
    async registerProviders(providers) {
        const promises = providers.map(Provider => {
            const provider = new Provider();
            provider.registerAttributes();
            if (provider.register)
                return provider.register();
            return null;
        });
        await Promise.all(promises);
    }
    /**
     * Preload all the files configured inside config/app
     * file.
     *
     * @private
     */
    async preloadFiles() {
        const preloads = await Promise.all(Config.get('app.preloads'));
        const promises = preloads.map(preload => {
            preload = path_1.normalize(preload);
            const { dir, name } = path_1.parse(utils_1.Path.config(preload));
            this.logger.success(`Preloading ${name} file`);
            return Promise.resolve().then(() => __importStar(require(`${dir}/${name}${this.extension}`)));
        });
        await Promise.all(promises);
    }
}
exports.Ignite = Ignite;
